<?php
namespace tests\Zeedhi\Framework\DBAL\Driver\OCI8;

use Doctrine\Common\EventManager;
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DriverManager;
use Doctrine\DBAL\Event\Listeners\OracleSessionInit;

class OCI8StatementTest extends \PHPUnit_Framework_TestCase {

    /** @var Connection */
    private $connection;

    protected function setUp() {
        $this->markTestSkipped("server 55 died");
        if (!extension_loaded('oci8')) {
            $this->markTestSkipped('oci8 is not installed.');
        }

        parent::setUp();

        $eventManager = new EventManager();
        $eventManager->addEventSubscriber(new OracleSessionInit());
        $this->connection = DriverManager::getConnection(array(
            "driver"      => 'oci8',
            "driverClass" => '\Zeedhi\Framework\DBAL\Driver\OCI8\Driver',
            "host"        => '192.168.122.55',
            "port"        => '1521',
            "user"        => 'ZEEDHI_DEMO',
            "password"    => 'teknisa',
            "dbname"      => 'xe',
        ), null, $eventManager);
        $this->connection->beginTransaction();
    }

    protected function tearDown() {
        return; //$this->markTestSkipped("55 server died");
        $this->connection->rollBack();
    }

    public function testSelectLongRaw() {
        $rows = $this->connection->fetchAll('SELECT * FROM TEST_LONG_RAW ORDER BY ID');
        $this->assertCount(9, $rows);
        $lengthByRowIdx = array(3, 3, 3, 6, 5, 6032, 6051, 7, 6032);
        foreach ($lengthByRowIdx as $idx => $length) {
            $this->assertArrayHasKey($idx, $rows);
            $this->assertArrayHasKey("TEXT", $rows[$idx]);
            $this->assertEquals($length, strlen($rows[$idx]["TEXT"]));
        }
    }

    public function testInsertLongRaw() {
        $stmt = $this->connection->prepare("INSERT INTO TEST_LONG_RAW VALUES (:ID, :TEXT)");
        $text = str_pad("TEST: ", 7500, ' THIS IS A VERY LARGE STRING.');
        $stmt->bindValue('ID', 10);
        $stmt->bindParam('TEXT', $text, SQLT_LBI);
        $this->assertTrue($stmt->execute());
        $valueInDb = $this->connection->fetchColumn("SELECT TEXT FROM TEST_LONG_RAW WHERE ID = :ID", array("ID" => 10));
        $this->assertEquals($text, $valueInDb);
    }
}
